/***************************************************************************
 * random_sound.cpp  -  random sounds support
 *
 * Copyright (C) 2008 Florian Richter
 ***************************************************************************/
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.
   
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../audio/random_sound.h"
#include "../core/game_core.h"
#include "../core/camera.h"
#include "../core/framerate.h"
#include "../audio/audio.h"
#include "../video/renderer.h"
#include "../input/mouse.h"
#include "../core/math/utilities.h"

/* *** *** *** *** *** cRandom_Sound *** *** *** *** *** *** *** *** *** *** *** */

cRandom_Sound :: cRandom_Sound( void )
: cSprite()
{
	// Set defaults
	cRandom_Sound::Init();
}

cRandom_Sound :: cRandom_Sound( XMLAttributes &attributes )
: cSprite()
{
	cRandom_Sound::Init();
	cRandom_Sound::Create_from_Stream( attributes );
}

cRandom_Sound :: ~cRandom_Sound( void )
{

}

void cRandom_Sound :: Create_from_Stream( XMLAttributes &attributes )
{
	// filename
	Set_Filename( attributes.getValueAsString( "file" ).c_str() );
	// position
	Set_Pos( static_cast<float>(attributes.getValueAsInteger( "pos_x" )), static_cast<float>(attributes.getValueAsInteger( "pos_y" )), 1 );
	// delay
	Set_Delay_Min( attributes.getValueAsInteger( "delay_min", delay_min ) );
	Set_Delay_Max( attributes.getValueAsInteger( "delay_max", delay_max ) );
	// volume
	Set_Volume_Min( attributes.getValueAsFloat( "volume_min", volume_min ) );
	Set_Volume_Max( attributes.getValueAsFloat( "volume_max", volume_max ) );
	// volume reduction
	Set_Volume_Reduction_Begin( attributes.getValueAsFloat( "volume_reduction_begin", volume_reduction_begin ) );
	Set_Volume_Reduction_End( attributes.getValueAsFloat( "volume_reduction_end", volume_reduction_end ) );
}

void cRandom_Sound :: Save_to_Stream( ofstream &ofile )
{
	ofile << "\t<sound>" << std::endl;

	// filename
	ofile << "\t\t<Property name=\"file\" value=\"" << filename.c_str() << "\" />" << std::endl;
	// position
	ofile << "\t\t<Property name=\"pos_x\" value=\"" << static_cast<int>(startposx) << "\" />" << std::endl;
	ofile << "\t\t<Property name=\"pos_y\" value=\"" << static_cast<int>(startposy) << "\" />" << std::endl;
	// delay
	ofile << "\t\t<Property name=\"delay_min\" value=\"" << delay_min << "\" />" << std::endl;
	ofile << "\t\t<Property name=\"delay_max\" value=\"" << delay_max << "\" />" << std::endl;
	// volume
	ofile << "\t\t<Property name=\"volume_min\" value=\"" << volume_min << "\" />" << std::endl;
	ofile << "\t\t<Property name=\"volume_max\" value=\"" << volume_max << "\" />" << std::endl;
	// volume reduction
	ofile << "\t\t<Property name=\"volume_reduction_begin\" value=\"" << volume_reduction_begin << "\" />" << std::endl;
	ofile << "\t\t<Property name=\"volume_reduction_end\" value=\"" << volume_reduction_end << "\" />" << std::endl;

	ofile << "\t</sound>" << std::endl;
}

void cRandom_Sound :: Init( void )
{
	sprite_array = ARRAY_ACTIVE;
	type = TYPE_SOUND;
	massivetype = MASS_PASSIVE;
	editor_posz = 0.111f;
	player_range = 0;
	name = "Sound";

	rect.w = 10;
	rect.h = 10;
	col_rect.w = rect.w;
	col_rect.h = rect.h;
	start_rect.w = rect.w;
	start_rect.h = rect.h;

	// default values
	delay_min = 1000;
	delay_max = 5000;
	volume_min = 90;
	volume_max = 100;
	volume_reduction_begin = 400;
	volume_reduction_end = 1000;
	
	distance_to_camera = 0;
	next_play_delay = 0;

	editor_color_volume_reduction_begin = Color( 0.1f, 0.5f, 0.1f, 0.2f );
	editor_color_volume_reduction_end = Color( 0.2f, 0.4f, 0.1f, 0.2f );
}

cRandom_Sound *cRandom_Sound :: Copy( void )
{
	cRandom_Sound *random_sound = new cRandom_Sound();
	random_sound->Set_Pos( startposx, startposy, 1 );
	random_sound->Set_Filename( filename.c_str() );
	random_sound->Set_Delay_Min( delay_min );
	random_sound->Set_Delay_Max( delay_max );
	random_sound->Set_Volume_Min( volume_min );
	random_sound->Set_Volume_Max( volume_max );
	random_sound->Set_Volume_Reduction_Begin( volume_reduction_begin );
	random_sound->Set_Volume_Reduction_End( volume_reduction_end );

	return random_sound;
}

void cRandom_Sound :: Set_Filename( string str )
{
	filename = str;
}

string cRandom_Sound :: Get_Filename( void ) const
{
	return filename;
}

void cRandom_Sound :: Set_Delay_Min( unsigned int delay )
{
	if( delay < 20 )
	{
		delay = 20;
	}

	delay_min = delay;
}

unsigned int cRandom_Sound :: Get_Delay_Min( void ) const
{
	return delay_min;
}

void cRandom_Sound :: Set_Delay_Max( unsigned int delay )
{
	if( delay < 20 )
	{
		delay = 20;
	}
	else if( delay < delay_min )
	{
		delay = delay_min;
	}

	delay_max = delay;
}

unsigned int cRandom_Sound :: Get_Delay_Max( void ) const
{
	return delay_max;
}

void cRandom_Sound :: Set_Volume_Min( float volume )
{
	if( volume < 0.1f )
	{
		volume = 0.1f;
	}
	else if( volume > volume_max )
	{
		volume = volume_max;
	}
	else if( volume > 100 )
	{
		volume = 100;
	}

	volume_min = volume;
}

float cRandom_Sound :: Get_Volume_Min( void ) const
{
	return volume_min;
}

void cRandom_Sound :: Set_Volume_Max( float volume )
{
	if( volume < 0.1f )
	{
		volume = 0.1f;
	}
	else if( volume < volume_min )
	{
		volume = volume_min;
	}
	else if( volume > 100 )
	{
		volume = 100;
	}

	volume_max = volume;
}

float cRandom_Sound :: Get_Volume_Max( void ) const
{
	return volume_max;
}

void cRandom_Sound :: Set_Volume_Reduction_Begin( float distance )
{
	if( distance < 1 )
	{
		distance = 1;
	}
	else if( distance > volume_reduction_end )
	{
		distance = volume_reduction_end;
	}

	volume_reduction_begin = distance;
}

float cRandom_Sound :: Get_Volume_Reduction_Begin( void ) const
{
	return volume_reduction_begin;
}

void cRandom_Sound :: Set_Volume_Reduction_End( float distance )
{
	if( distance < 1 )
	{
		distance = 1;
	}
	else if( distance < volume_reduction_begin )
	{
		distance = volume_reduction_begin;
	}

	volume_reduction_end = distance;
}

float cRandom_Sound :: Get_Volume_Reduction_End( void ) const
{
	return volume_reduction_end;
}

void cRandom_Sound :: Update( void )
{
	Update_Valid_Update();

	if( !valid_update )
	{
		return;
	}

	// subtract duration of this frame in milliseconds from next play counter
	next_play_delay -= ( pFramerate->speedfactor * 1000 ) / static_cast<float>(DESIRED_FPS);

	// play it
	if( next_play_delay <= 0 )
	{
		// set random volume
		float sound_volume = volume_min;

		if( volume_max > volume_min )
		{
			sound_volume += Get_Random_Float( 0, volume_max - volume_min );
		}

		sound_volume *= 0.01f;

		// adjust volume based on distance
		if( distance_to_camera > volume_reduction_begin )
		{
			float distance = distance_to_camera - volume_reduction_begin;
			sound_volume *= ( 1.0f - distance / ( volume_reduction_end - volume_reduction_begin ) );
		}

		// play sound
		pAudio->Play_Sound( filename, -1, static_cast<int>(sound_volume * static_cast<float>(MIX_MAX_VOLUME)) );

		// set next delay
		next_play_delay = static_cast<float>(delay_min);

		if( delay_max > delay_min )
		{
			next_play_delay += Get_Random_Float( 0, static_cast<float>(delay_max - delay_min) );
		}
	}
}

void cRandom_Sound :: Draw( cSurfaceRequest *request /* = NULL */ )
{
	if( !valid_draw )
	{
		return;
	}

	// draw color rect
	pVideo->Draw_Rect( col_rect.x - pActive_Camera->x, col_rect.y - pActive_Camera->y, col_rect.w, col_rect.h, editor_posz, &greenyellow );
	// volume reduction begin
	cCircleRequest *circle_request = new cCircleRequest();
	pVideo->Draw_Circle( col_rect.x - pActive_Camera->x, col_rect.y - pActive_Camera->y, volume_reduction_begin, editor_posz - 0.0001f, &editor_color_volume_reduction_begin, circle_request );
	circle_request->line_width = 3;
	// add request
	pRenderer->Add( circle_request );
	// volume reduction end
	circle_request = new cCircleRequest();
	pVideo->Draw_Circle( col_rect.x - pActive_Camera->x, col_rect.y - pActive_Camera->y, volume_reduction_end, editor_posz - 0.0002f, &editor_color_volume_reduction_end, circle_request  );
	circle_request->line_width = 3;
	// add request
	pRenderer->Add( circle_request );
}

bool cRandom_Sound :: Is_Update_Valid( void )
{
	// get distance from camera center position
	float dx = fabs( pActive_Camera->x + (GAME_RES_W * 0.5f) - posx );
	float dy = fabs( pActive_Camera->y + (GAME_RES_H * 0.5f) - posy );

	distance_to_camera = sqrt( dx * dx + dy * dy );

	// if outside the zone
	if( distance_to_camera >= volume_reduction_end )
	{
		return 0;
	}

	return 1;
}

bool cRandom_Sound :: Is_Draw_Valid( void )
{
	// if editor not enabled
	if( !editor_enabled )
	{
		return 0;
	}

	// not visible on the screen and not mouse object
	if( !Is_Visible_on_Screen() && pMouseCursor->active_object != this )
	{
		return 0;
	}

	return 1;
}

void cRandom_Sound :: Editor_Activate( void )
{
	WindowManager &wmgr = WindowManager::getSingleton();

	// image
	Editbox *editbox = static_cast<Editbox *>(wmgr.createWindow( "TaharezLook/Editbox", "editor_sound_filename" ));
	editbox->setTooltipText( "Filename" );
	Editor_Add( "Filename", editbox, 200 );

	editbox->setText( filename.c_str() );
	editbox->subscribeEvent( Editbox::EventTextChanged, Event::Subscriber( &cRandom_Sound::Editor_Filename_Key, this ) );

	// delay min
	editbox = static_cast<Editbox *>(wmgr.createWindow( "TaharezLook/Editbox", "editor_sound_delay_min" ));
	editbox->setTooltipText( "Delay Minimum" );
	Editor_Add( "Delay Minimum", editbox, 90 );

	editbox->setText( int_to_string( delay_min ) );
	editbox->subscribeEvent( Editbox::EventKeyUp, Event::Subscriber( &cRandom_Sound::Editor_Delay_Min_Key, this ) );

	// delay max
	editbox = static_cast<Editbox *>(wmgr.createWindow( "TaharezLook/Editbox", "editor_sound_delay_max" ));
	editbox->setTooltipText( "Delay Maximum" );
	Editor_Add( "Maximum", editbox, 90, 28, 0 );

	editbox->setText( int_to_string( delay_max ) );
	editbox->subscribeEvent( Editbox::EventKeyUp, Event::Subscriber( &cRandom_Sound::Editor_Delay_Max_Key, this ) );

	// volume min
	editbox = static_cast<Editbox *>(wmgr.createWindow( "TaharezLook/Editbox", "editor_sound_volume_min" ));
	editbox->setTooltipText( "Volume Minimum" );
	Editor_Add( "Volume Minimum", editbox, 90 );

	editbox->setText( int_to_string( static_cast<int>(volume_min) ) );
	editbox->subscribeEvent( Editbox::EventKeyUp, Event::Subscriber( &cRandom_Sound::Editor_Volume_Min_Key, this ) );

	// volume max
	editbox = static_cast<Editbox *>(wmgr.createWindow( "TaharezLook/Editbox", "editor_sound_volume_max" ));
	editbox->setTooltipText( "Volume Maximum" );
	Editor_Add( "Maximum", editbox, 90, 28, 0 );

	editbox->setText( int_to_string( static_cast<int>(volume_max) ) );
	editbox->subscribeEvent( Editbox::EventKeyUp, Event::Subscriber( &cRandom_Sound::Editor_Volume_Max_Key, this ) );

	// volume reduction begin
	editbox = static_cast<Editbox *>(wmgr.createWindow( "TaharezLook/Editbox", "editor_sound_volume_reduction_begin" ));
	editbox->setTooltipText( "Volume Reduction Begin" );
	Editor_Add( "Volume Reduction Begin", editbox, 90 );

	editbox->setText( int_to_string( static_cast<int>(volume_reduction_begin) ) );
	editbox->subscribeEvent( Editbox::EventKeyUp, Event::Subscriber( &cRandom_Sound::Editor_Volume_Reduction_Begin_Key, this ) );

	// volume reduction end
	editbox = static_cast<Editbox *>(wmgr.createWindow( "TaharezLook/Editbox", "editor_sound_volume_reduction_end" ));
	editbox->setTooltipText( "Volume Reduction End" );
	Editor_Add( "End", editbox, 90, 28, 0 );

	editbox->setText( int_to_string( static_cast<int>(volume_reduction_end) ) );
	editbox->subscribeEvent( Editbox::EventKeyUp, Event::Subscriber( &cRandom_Sound::Editor_Volume_Reduction_End_Key, this ) );

	// init
	Editor_Init();
}

bool cRandom_Sound :: Editor_Filename_Key( const EventArgs &event )
{
	const WindowEventArgs &windowEventArgs = static_cast<const WindowEventArgs&>( event );
	string str_text = static_cast<Editbox *>( windowEventArgs.window )->getText().c_str();

	Set_Filename( str_text );

	return 1;
}

bool cRandom_Sound :: Editor_Delay_Min_Key( const EventArgs &event )
{
	const WindowEventArgs &windowEventArgs = static_cast<const WindowEventArgs&>( event );
	string str_text = static_cast<Editbox *>( windowEventArgs.window )->getText().c_str();

	Set_Delay_Min( string_to_int( str_text ) );

	return 1;
}

bool cRandom_Sound :: Editor_Delay_Max_Key( const EventArgs &event )
{
	const WindowEventArgs &windowEventArgs = static_cast<const WindowEventArgs&>( event );
	string str_text = static_cast<Editbox *>( windowEventArgs.window )->getText().c_str();

	Set_Delay_Max( string_to_int( str_text ) );

	return 1;
}

bool cRandom_Sound :: Editor_Volume_Min_Key( const EventArgs &event )
{
	const WindowEventArgs &windowEventArgs = static_cast<const WindowEventArgs&>( event );
	string str_text = static_cast<Editbox *>( windowEventArgs.window )->getText().c_str();

	Set_Volume_Min( string_to_float( str_text ) );

	return 1;
}

bool cRandom_Sound :: Editor_Volume_Max_Key( const EventArgs &event )
{
	const WindowEventArgs &windowEventArgs = static_cast<const WindowEventArgs&>( event );
	string str_text = static_cast<Editbox *>( windowEventArgs.window )->getText().c_str();

	Set_Volume_Max( string_to_float( str_text ) );

	return 1;
}

bool cRandom_Sound :: Editor_Volume_Reduction_Begin_Key( const EventArgs &event )
{
	const WindowEventArgs &windowEventArgs = static_cast<const WindowEventArgs&>( event );
	string str_text = static_cast<Editbox *>( windowEventArgs.window )->getText().c_str();

	Set_Volume_Reduction_Begin( string_to_float( str_text ) );

	return 1;
}

bool cRandom_Sound :: Editor_Volume_Reduction_End_Key( const EventArgs &event )
{
	const WindowEventArgs &windowEventArgs = static_cast<const WindowEventArgs&>( event );
	string str_text = static_cast<Editbox *>( windowEventArgs.window )->getText().c_str();

	Set_Volume_Reduction_End( string_to_float( str_text ) );

	return 1;
}
